.text-bold {
    font-weight: 600;
}

.library-box {
    background-color: transparent;
}

.library-box:selected .library-btn {
	background-color: transparent;
}

.library-box:selected .library-btn image {
	color: @fg_color;
}

.library-inner-box {
    background: transparent;
    transition: background 320ms ease;
    color: @fg_color;
}

.library-inner-box .library-btn {
    opacity: 0;
}

.library-inner-box:hover {
    background: shade (@bg_color, 0.9);
}

.library-inner-box:hover .library-btn {
    opacity: 1;
}

.library-btn:active,
.library-btn:focus {
    background-color: transparent;
}

.grid-motion {
	background-color: alpha (#3689e6, 0.25);
    border-radius: 4px;
}

.library-titlebar {
    border-bottom: 1px solid shade (@bg_color, 0.9);
    background: shade (@bg_color, 0.95);
}

.library-toolbar {
    background: shade (@bg_color, 0.95);
    border-top: 1px solid shade (@bg_color, 0.9);
}

.library-colorbox {
    border-radius: 12px;
    box-shadow: inset 0px 0px 0px 1px rgba(0, 0, 0, 0.2);
}

.toolbar-btn {
    background: transparent;
    border: none;
    box-shadow: none;
}

.toolbar-btn.active {
    background: shade (@bg_color, 0.9);
    opacity: 1;
}

.headerbar-separator {
    margin-right: 3px;
    margin-left: 3px;
}

infobar.inline revealer > box {
    box-shadow: inset 0 -1px 0 0 alpha (#fff, 0.4);
    border-width: 1px;
    border-radius: 4px;
}

.color-button radio,
.color-button radio:checked {
    border-color: alpha (#000, 0.3);
    box-shadow:
        inset 0 1px 0 0 alpha (@inset_dark_color, 0.7),
        inset 0 0 0 1px alpha (@inset_dark_color, 0.3),
        0 1px 0 0 alpha (@bg_highlight_color, 0.3);
    padding: 10px;
    -gtk-icon-shadow: none;
}

.color-button radio:focus {
    border-color: @colorAccent;
    box-shadow:
        inset 0 1px 0 0 alpha (@inset_dark_color, 0.7),
        inset 0 0 0 1px alpha (@inset_dark_color, 0.3),
        inset 0 0 0 1px alpha (@bg_highlight_color, 0.05),
        0 1px 0 0 alpha (@bg_highlight_color, 0.3),
        0 0 0 1px alpha (@colorAccent, 0.25);
}

.color-dark radio {
    background: #252E32;
    border-color: #151B1C;
    color: #fff;
}

.color-light radio {
    background: #fdf6e3;
    color: #4d4d4d;
}

.color-white radio {
    background: #fff;
    color: #000;
}

.query-error {
    background-color: alpha (@STRAWBERRY_300, 0.2);
    border: 1px solid @STRAWBERRY_500;
    border-radius: 4px;
}

button.notebook-temp-fix image {
    color: @selected_fg_color;
}

.database-panel-overlay {
    background-color: alpha (#000, 0.3);
}

.database-panel {
    border-radius: 3px;
    border: 1px solid alpha (#000, 0.3);
    box-shadow: 0 3px 1px -2px rgba(0,0,0,.2),
                0 2px 2px 0 rgba(0,0,0,.14),
                0 1px 5px 0 rgba(0,0,0,.12);
    text-shadow: none;
    background-color: shade (@base_color, 1.15);
}

.database-panel-bottom {
    background-color: alpha (#000, 0.1);
    box-shadow: inset 0 5px 5px -10px #000
}
